﻿'***********************************************************
'
'[histroy]
'           2012/8/21   URLから“&elm=minutes”を取った。
'           2013/12/7   気圧の計測を行っていない観測点に対応
'           2014/1/19   1時間毎の観測データのURLに対応
'***********************************************************
Imports System.Text.RegularExpressions

''' <summary>
''' 
''' </summary>
''' <remarks></remarks>
Public Class Amedas
    ' 構造体宣言
    Public Structure Amedas
        ' ***********メンバ変数*********************
        ''' <summary>
        ''' ID
        ''' <para>ブロックNo</para>
        ''' </summary>
        ''' <remarks></remarks>
        Public ID As String
        ''' <summary>
        ''' 地点名
        ''' </summary>
        ''' <remarks></remarks>
        Public name As String
        ''' <summary>
        ''' Prec No.
        ''' <para>都道府県・観測管轄所番号</para>
        ''' </summary>
        ''' <remarks></remarks>
        Public precNo As Integer
        ''' <summary>
        ''' url_body
        ''' <para>URLのドメインを含む基部</para>
        ''' </summary>
        ''' <remarks></remarks>
        Private url_body As String
        ''' <summary>
        ''' url_part
        ''' <para>URLの一部</para>
        ''' <para>ブロックIDによって決定される.</para>
        ''' </summary>
        ''' <remarks></remarks>
        Private url_part As String
        ' ***********メソッド*********************
        ''' <summary>
        ''' 10分毎の観測データへのアクセスに必要なURLを作成する
        ''' </summary>
        ''' <param name="t"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function GetURL_10min(ByVal t As Date)
            Return Me.url_body + _
                "10min_" + _
                Me.url_part + "1.php?prec_no=" + _
                Me.precNo.ToString() + "&block_no=" + _
                Me.ID + "&year=" + t.Year.ToString + "&month=" + t.Month.ToString("00") + "&day=" + t.Day.ToString("00") + "&view=" ' URL作成
        End Function
        ''' <summary>
        ''' 1時間毎の観測データへのアクセスに必要なURLを作成する
        ''' </summary>
        ''' <param name="t"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function GetURL_hourly(ByVal t As Date)
            Return Me.url_body + _
                "hourly_" + _
                Me.url_part + "1.php?prec_no=" + _
                Me.precNo.ToString() + "&block_no=" + _
                Me.ID + "&year=" + t.Year.ToString + "&month=" + t.Month.ToString("00") + "&day=" + t.Day.ToString("00") + "&view=" ' URL作成
        End Function
        ''' <summary>
        ''' アメダスのURLから基地局IDを取得する
        ''' </summary>
        ''' <param name="url">URLの文字列</param>
        ''' <returns>取得に成功すればIDが返ります。失敗すれば0が返されます。</returns>
        ''' <remarks></remarks>
        Public Function GetAmedasId(ByVal url As String) As Integer
            Dim id As Integer
            Dim success As Boolean = False
            Dim r As New Regex("block_no=\d+", RegexOptions.IgnoreCase)     ' Regexオブジェクトを作成 
            Dim m As Match = r.Match(url)                                   ' url文字列内で正規表現と一致する対象を1つ検索 

            If m.Success Then                                               ' もし、ヒットしたら処理を行う
                success = Integer.TryParse(m.Value.Split("=")(1), id)       ' スプリットで文字列を切り出して、IDのみを抽出
            End If
            If success = False Then id = 0 ' 失敗していた時の処置
            GetAmedasId = id
        End Function
        ''' <summary>
        ''' アメダスのURLから基地局IDを取得する
        ''' </summary>
        ''' <param name="url">URLの文字列</param>
        ''' <returns>取得に成功すればIDが返ります。失敗すれば0が返されます。</returns>
        ''' <remarks></remarks>
        Public Function GetPrecNo(ByVal url As String) As Integer
            Dim precNo As Integer
            Dim success As Boolean = False
            Dim r As New Regex("prec_no=\d+", RegexOptions.IgnoreCase)      ' Regexオブジェクトを作成 
            Dim m As Match = r.Match(url)                                   ' url文字列内で正規表現と一致する対象を1つ検索 

            If m.Success Then                                               ' もし、ヒットしたら処理を行う
                success = Integer.TryParse(m.Value.Split("=")(1), precNo)   ' スプリットで文字列を切り出して、IDのみを抽出
            End If
            If success = False Then precNo = 0 ' 失敗していた時の処置
            GetPrecNo = precNo
        End Function
        ''' <summary>
        ''' コンストラクタ
        ''' </summary>
        ''' <param name="_ID">ブロックNo</param>
        ''' <param name="_name">地点名</param>
        ''' <param name="_precNo">Prec No.</param>
        ''' <remarks></remarks>
        Sub New(ByRef _ID As String, ByVal _name As String, ByVal _precNo As Integer)
            Me.ID = _ID
            Me.name = _name
            Me.precNo = _precNo


            Me.url_body = "http://www.data.jma.go.jp/obd/stats/etrn/view/"

            Dim i As Integer = Integer.Parse(_ID)
            If i < 10000 Then
                Me.url_part = "a"
            Else
                Me.url_part = "s"
            End If
        End Sub
    End Structure

End Class
